#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import time
from datetime import datetime, timedelta


def main():
    import logging
    from random import randint
    log = logging.getLogger(__name__)
    logging.basicConfig(level=logging.DEBUG, format='%(levelname)-8s %(message)s')

    rp = RePrint()
    for i in xrange(10):
        rp(u'мессага')
        time.sleep(0.1)
    rp.clear()
    
    for i in progress(xrange(10), logger=log.info):
        print i
        time.sleep(0.1)
        
    CNT = 10
    cnt = Progress(CNT, inline=True)
    for i in xrange(CNT):
        cnt.put(u'мессага' + ' ' + '.' * randint(1, 10))
        time.sleep(randint(1, 3) / 10.0)


class Timer(object):
    '''
    Таймер
    '''
    def __init__(self):
        self.start = time.time()
        
    def sec(self):
        return time.time() - self.start
    
    def __str__(self):
        return sec2str(self.sec())

def progress(objs, *args, **kwargs):
    '''
    Счётчик-генератор
    '''
    c = Progress(len(objs), *args, **kwargs)
    for obj in objs:
        c.put()
        yield obj

def sec2str(sec):
    '''
    Секунды в удобочитаемом вормате
    '''
    return '%2.2f s.' % sec if sec < 60 else str(
        timedelta(seconds=int(sec)))

class Progress(object):
    '''
    Счётчик
    '''

    def __init__(self, limit, step=None, start=1, inline=False, logger=None):
        self.limit = limit
        self.step = step if step else int(limit / 100) or 1
        self.cnt = start
        self.cnt_tpl = '%%(cnt_cur)%(cnt_len)si / %%(cnt_limit)%(cnt_len)si |%%(cnt_perc)3i%%%% | %%(time_step)8s | ~%%(time_left)8s | %%(speed)s/sec' % {
            'cnt_len' : len(str(limit))}
        self.tm = Timer()
        self.full_tm = Timer()
        self.logger = logger
        self.inline = not logger and inline
        self.last_msg_len = 0
        
    def log(self, msg):
        if self.logger:
            self.logger(msg)
        elif self.inline:
            if self.last_msg_len:
                #~ sys.stdout.write('\x08' * self.last_msg_len)
                sys.stdout.write('\x0D')
            sys.stdout.write(msg)
            overflow = self.last_msg_len - len(msg)
            if overflow > 0:
                sys.stdout.write(' ' * overflow)
            self.last_msg_len = len(msg)
        else:
            sys.stdout.write(msg)
            sys.stdout.write('\n')

    def put(self, msg=''):
        '''
        Выводит счётчик если пройдено нужное количество шагов
        Использование:
            if cnt.put('дополнительные данные, может быть функция'):
                # сделать что-то, если вывелся счётчик
                ...
        '''
        res = False
        if not self.cnt % self.step or self.cnt == self.limit:
            res = True
            cnt_perc = float(self.cnt) / self.limit
            time_full = self.full_tm.sec()
            time_left = sec2str((1.0 / cnt_perc) * time_full - time_full)
            #~ cur_step_cnt = self.step if not self.cnt % self.step else self.cnt - self.limit
            #~ speed = cur_step_cnt / self.tm.sec()
            speed = self.cnt / time_full
            speed = '%0.2f' % speed if speed < 10 else '%i' % speed
            m = self.cnt_tpl % {   
                'cnt_cur': self.cnt,
                'cnt_limit': self.limit,
                'cnt_perc': cnt_perc * 100,
                'time_step': self.tm,
                'time_left': time_left,
                'speed': speed,
            }
            self.tm = Timer()
            if msg:
                msg = msg() if hasattr(msg, '__call__') else unicode(msg)
                m += ' | %s' % msg
            if self.cnt == self.limit:
                m += ' | full time: %s' % self.full_tm
                if self.inline:
                    m = m + '\n'
            self.log(m)
            sys.stdout.flush()
        self.cnt += 1
        return res


class RePrint(object):
    def __init__(self, step=1, counter=True, timer=True, dots=True):
        self.step = step
        self.last_msg = ''
        self.counter = counter
        self.tm = Timer() if timer else None
        self.cnt = 0
        self.dots = dots
    
    def clear(self):
        if self.cnt:
            self.put(' ' * len(self.last_msg))
    
    def __call__(self, msg):
        self.cnt += 1
        if not self.cnt % self.step:
            if self.tm:
                msg = ' %s | %s' % (self.tm, msg) 
            if self.counter:
                msg = ' %3s | %s' % (self.cnt, msg)
            if self.dots:
                msg += ' ' + '.' * (self.cnt % 5)
            self.clear()
            self.put(msg)
        
    def put(self, msg):
        msg = msg.encode('utf-8') if isinstance(msg, unicode) else str(msg)
        sys.stdout.write(msg)
        sys.stdout.write('\x0D')
        sys.stdout.flush()
        self.last_msg = msg

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
'''
Объектная обёртка над urllib
Пример:
    res = query('dev.pooqle.com', ref='ya.ru', get=dict(a=1), post=dict(b=2))
    print res.url   # урл может отличаться от запрашиваемого в случае редиректа
    print res.code
    print res.headers
    print res.body  # запрос тела страницы произойдёт только тут
    print bool(res) # True - если код == 200 и загрузилось body
'''
import os
import re
import sys
import random
import urllib
import urllib2
import httplib
import logging
from gzip import GzipFile
from cookielib import CookieJar, Cookie
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO
try:
    import cPickle as pickle
except ImportError:
    import pickle
import simplejson as json

try:
    import leaf
except ImportError:
    print 'warning: module not found: leaf'

from decorators import cached_property


url = sys.modules[__name__]  # for auto-import

RE_CONTENT_TYPE_TAG = re.compile(r'<meta[^>]+http-equiv\s*=\s*["\']Content-Type[^>]+', re.I|re.S)
RE_CHARSET = re.compile(r'charset\s*=\s*([-_a-z0-9]+)', re.I)


http_handler = urllib2.HTTPHandler()
http_handler.set_http_debuglevel(1)
try:
    https_handler = urllib2.HTTPSHandler()
    https_handler.set_http_debuglevel(1)
except AttributeError:
    pass
#~ logger = logging.getLogger()
#~ logger.addHandler(logging.StreamHandler(sys.stdout))
#~ logger.setLevel(logging.NOTSET)


def main():
    import doctest
    doctest.testmod()
    
    return
    url = 'dev.pooqle.com'

    res = load(url,
        #~ debug=1, 
        #~ log_dir='var/om.url.logs'
        )
    print res.code
    print res.body
    print res.load_defaults['user_agent']
    #~ res.load(url)
    #~ print res.code
    #~ print res.url
    #~ print res.cookies



def load(url, size=None, get=None, post=None, compress=True,
        headers=None, cookies=None, user_agent=None, referer=None,
        proxy=None, proxy_type='http',
        charset='utf-8', safe=True, debug=False, log_dir=None,
        _req_cnt=0):
    '''
    Получает страницу, с любым статусом
        **size = ограничение на размер скачиваемой части страницы в байтах
        **get = словарь или список пар переменных GET-запроса
        **post = словарь или список пар переменных POST-запроса
        **headers = словарь заголовков
        **cookies = StringCookieJar object or res.cj.dumps() string
        **user_agent = юзерагент
        **referer = реферер
        **proxy = прокси в виде user:passwd@host:port
        **proxy_type = тип прокси
        **compress = получение страниц сжатыми по возможности
        **safe = перехват всех исключений, возвращает Response(code=0, body='')
        **debug = лог запросов в консоль
        **log_dir = папка для сохранения страниц в debug-режиме
    '''
    url = url_fix(url, charset)
    # add get-params
    if get:
        if '#' in url:
            pos = url.find('#')
            url, end = url[:pos], url[pos:]
        else:
            end = ''
        sep = '&' if '?' in url else '?'
        url += sep + urllib.urlencode(encoded_unique_items(get)) + end
    q = {'url': url}
    if post:
        q['data'] = urllib.urlencode(encoded_unique_items(post))
    req = urllib2.Request(**q)
    if headers:
        for item in encoded_unique_items(headers):
            req.add_header(*item)
    if user_agent is None:
        user_agent = str(random.choice(USER_AGENTS)())
    req.add_header('User-agent', user_agent)
    if referer:
        req.add_header('Referer', url_fix(referer))
    if cookies is None:
        cookies = {}
    if isinstance(cookies, StringCookieJar):
        cookies = cookies
    elif isinstance(cookies, basestring):
        cookies = StringCookieJar(cookies)
    elif isinstance(cookies, dict):
        cj = StringCookieJar()
        for name, value in cookies.iteritems():
            cookie = Cookie(
                version=0,
                name=name,
                value=value,
                port=None,
                port_specified=False,
                domain=req.get_host(),
                domain_specified=False,
                domain_initial_dot=False,
                path='/',
                path_specified=True,
                secure=False,
                expires=None,
                discard=True,
                comment=None,
                comment_url=None,
                rest={'HttpOnly': None},
                rfc2109=False
            )
            cj.set_cookie(cookie)
        cookies = cj
    else:
        assert False, 'cookies must be StringCookieJar, dict or basestring type'
    handlers = [urllib2.HTTPCookieProcessor(cookies)]
    if compress:
        req.add_header("Accept-Encoding", "gzip")
    _req_cnt += 1
    if debug:
        print '<<< [%02i] %s %s' % (_req_cnt, 'POST' if post else 'GET', url)
        handlers.extend([http_handler, https_handler])
    if proxy:
        if '@' in proxy:
            schema = '%s://' % proxy_type
            if proxy.startswith(schema):
                proxy = proxy[len(schema):]
            proxy_user, proxy_host = proxy.split('@')
            proxy_passwd = None
            if ':' in proxy_user:
                proxy_user, proxy_passwd = proxy_user.split(':')
            proxy_handler = urllib2.ProxyHandler({proxy_type: proxy_host})
            proxy_auth_handler = urllib2.ProxyBasicAuthHandler(ProxyPasswordMgr())
            proxy_auth_handler.add_password(None, None, proxy_user, proxy_passwd)
            handlers.extend([proxy_handler, proxy_auth_handler])
        else:
            req.set_proxy(proxy, proxy_type)
    opener = urllib2.build_opener(*handlers)
    if safe:
        try:
            conn = opener.open(req)
        except urllib2.HTTPError, e:
            conn = e
        except Exception, ex:
            conn = StringIO('')
    else:
        conn = opener.open(req)
    load_defaults = {
        'compress': compress,
        'cookies': cookies,
        'user_agent': user_agent,
        'referer': getattr(conn, 'url', None),
        'proxy': proxy,
        'proxy_type': proxy_type,
        'safe': safe,
        'debug': debug,
        'log_dir': log_dir,
        '_req_cnt': _req_cnt,
    }
    return Response(conn, getattr(conn, 'code', 0), size, load_defaults)


class Response(object):
    body_load_error = False
    
    def __init__(self, conn, code, size, load_defaults):
        self.conn = conn
        self.size = size
        self.code = code
        self.load_defaults = load_defaults
        if self.load_defaults['log_dir']:
            self._log_page()
        
    def __nonzero__(self):
        self.body
        return self.code == 200 and not self.body_load_error

    def load(self, url, **kwargs):
        for k, v in self.load_defaults.iteritems():
            if k not in kwargs:
                kwargs[k] = v
        return load(url, **kwargs)

    @cached_property
    def url(self):
        return self.load_defaults['referer']

    @cached_property
    def headers(self):
        return self.conn.headers.dict if self.code else {}
    
    @cached_property
    def cookies(self):
        return self.load_defaults['cookies']
    
    @cached_property
    def body(self):
        try:
            if 'gzip' in self.headers.get('content-encoding', ''):
                ret = GzipFile(
                    fileobj=StringIO(self.conn.read(self.size))).read()
            else:
                ret = self.conn.read(self.size) if self.code else ''
        except Exception:
            if not self.load_defaults['safe']:
                raise
            self.body_load_error = True
            ret = ''
        return ret
            
    @cached_property
    def ubody(self):
        return self.body.decode(self.charset)
        
    @cached_property
    def dom(self):
        return leaf.parse(self.body)
    
    @cached_property
    def json(self):
        return json.loads(self.body)
    
    @cached_property
    def charset(self):
        enc = 'utf-8'
        # ищем в заголовках
        hdr = self.headers.get('Content-Type', '').lower()
        try:
            enc = hdr.split('charset')[1].split('=')[1].strip().lower()
        except IndexError:
            # в заголовках пусто, ищем в метатеге
            match = RE_CONTENT_TYPE_TAG.search(self.body)
            if match:
                match = RE_CHARSET.search(match.group(0))
                if match:
                    enc = match.group(1).lower()
        return enc
        
    def form(self, items=None, form_num=None):
        '''
        *items  : dict or items данных для заполнения формы,
        return form dict
        '''
        items = items or ()
        if not isinstance(form_num, (int, type(None))):
            form = form_num
        else:
            forms = self.dom.forms
            if form_num is None:
                if len(forms) > 1:
                    # form is not set, choose the form of maximum size
                    forms.sort(key=lambda x: -len(x.fields))
                form = forms[0]
            else:
                form = forms[form_num]
        ret = dict(form.form_values())
        ret.update(dict(items) if not isinstance(items, dict) else items)
        return ret
        
    def _log_page(self):
        dir = self.load_defaults['log_dir']
        cnt = self.load_defaults['_req_cnt']
        if cnt == 1:
            # prepare log directory
            if not os.path.exists(dir):
                os.makedirs(dir)                
            for fn in os.listdir(dir):
                full_fn = os.path.join(dir, fn)
                if os.path.isfile(full_fn) and fn.split('.')[0].isdigit():
                    os.remove(full_fn)
        fn_tpl = os.path.join(dir, '%02i.%%s' % cnt)
        open(fn_tpl % 'hdrs.txt', 'w').write(str(self.conn.headers))
        ct = self.headers.get('content-type', '').lower()
        ext = {
            'image/jpeg': 'jpg',
            'image/gif': 'gif',
            'image/png': 'png',
        }.get(ct, 'html')
        open(fn_tpl % ext, 'w').write(self.body)                    


class ProxyPasswordMgr:
    def __init__(self):
        self.user = self.passwd = None
    def add_password(self, realm, uri, user, passwd):
        self.user = user
        self.passwd = passwd
    def find_user_password(self, realm, authuri):
        return self.user, self.passwd


class StringCookieJar(CookieJar):
    def __init__(self, dump=None, policy=None):
        CookieJar.__init__(self, policy)
        if dump:
            if isinstance(dump, unicode):
                dump = dump.encode('utf-8')
            self._cookies = pickle.loads(dump)

    def dumps(self):
        return pickle.dumps(self._cookies)


def encoded_unique_items(items_or_dict, charset='utf-8'):
    if isinstance(items_or_dict, dict):
        items_or_dict = items_or_dict.iteritems()
    ret = {}
    for k, v in items_or_dict:
        ret[(
            k.encode(charset) if isinstance(k, unicode) else k,
            v.encode(charset) if isinstance(v, unicode) else v
        )] = True
    return ret.keys()


def url_fix(url, charset='utf-8'):
    '''
        Fix no path:
            >>> url_fix('host.com')
            'http://host.com/'
            
            >>> url_fix('host.com?a=b')
            'http://host.com/?a=b'
            
            >>> url_fix('host.com#a=b')
            'http://host.com/#a=b'
            
            >>> url_fix('https://host.com?a=b&a=c#id')
            'https://host.com/?a=b&a=c#id'
            
        Fix spaces:
            >>> url_fix('en.wikipedia.org/wiki/Doorway page')
            'http://en.wikipedia.org/wiki/Doorway%20page'
            
        Fix unicode:
            >>> url_fix('ru.wikipedia.org/wiki/Питон'.decode('utf-8'))
            'http://ru.wikipedia.org/wiki/%D0%9F%D0%B8%D1%82%D0%BE%D0%BD'
        
        Fix idna host:
            >>> url_fix('рег.рф/aaa/'.decode('utf-8'))
            'http://xn--c1ad6a.xn--p1ai/aaa/'
    '''
    url = url.strip()
    if not url.startswith(('http://', 'https://', 'ftp://', 'ftps://')):
        # no schema
        url = 'http://' + url
    if '/' not in url.split('//')[-1]:
        # no path
        frags = [url.find(sep) for sep in ('?', '#') if sep in url]
        if frags:
            x = min(frags)
            host, qs = url[:x], url[x:]
        else:
            host, qs = url, ''
        url = host + '/' + qs
    if isinstance(url, unicode):
        url = url.encode(charset)
    host = url.split('//', 1)[1].split('/', 1)[0]
    if not host.replace('.', '').replace('-', '').isalnum():
        # idn
        host = host.decode('utf-8').encode('idna')
        schema = url.split('//', 1)[0]
        uri = url.split('/', 3)[-1]
        url = '%s//%s/%s' % (schema, host, uri)
    url = urllib.quote(url, safe="%/:=&?~#+!$,;'@()*[]")
    return url


class RNDDict(object):
    TPL = None
    DATA = {}
    
    def __getitem__(self, key):
        return random.choice(self.DATA[key])

    def __str__(self):
        return self.TPL % self


class UAOpera(RNDDict):
    # http://www.useragentstring.com/pages/Opera/
    TPL = '%(name)s (%(os)s; U; %(lang)s) Presto/%(presto)s Version/%(ver)s'
    DATA = {
        'name': ['Opera/9.80'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'lang': ['en', 'en-US', 'ru', 'de'],
        'presto': ['2.8.99', '2.8.131', '2.7.62'],
        'ver': ['11.10', '11.01'],
    }

class UAFirefox(RNDDict):
    TPL = '%(name)s (Windows; U; %(os)s; %(lang)s) Gecko/%(engine)s Firefox/%(ver)s'
    DATA = {
        'name': ['Mozilla/5.0'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'lang': ['en', 'en-US', 'ru', 'de'],
        'engine': ['20100401', '20091204', '20090402', '20091124', '20100824', '20100722'],
        'ver': ['4.0', '3.8', '3.6.9', '3.6.8', '3.6.7', '3.6.6', '3.6.5', '3.6.3', '3.6.2', '3.6.16', '3.6.15', '3.6.14', '3.6.13', '3.6.12',],
        'rv': ['1.9.2.8', '1.9.2.3', '1.9.2.9', '', '', ],
    }

class UAChrome(RNDDict):
    TPL = '%(name)s (%(os)s) AppleWebKit/%(engine)s (KHTML, like Gecko) Chrome/%(ver)s'
    DATA = {
        'name': ['Mozilla/5.0'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'engine': ['534.25', '534.24'],
        'ver': ['12.0.706.0 Safari/534.25', '12.0.702.0 Safari/534.24', '11.0.699.0 Safari/534.24', '11.0.697.0 Safari/534.24'],
    }

class UAIE8(RNDDict):
    TPL = '%(ua)s'
    DATA = {
        'ua': [s.strip() for s in '''
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 3.0.04506.30)
            Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0; Trident/4.0; FBSMTWB; .NET CLR 2.0.34861; .NET CLR 3.0.3746.3218; .NET CLR 3.5.33652; msn OptimizedIE8;ENUS)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.3; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.2)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 3.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; msn OptimizedIE8;ZHCN)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; .NET4.0C; .NET4.0E; Zune 4.7)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 4.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Zune 4.7)
        '''.split('\n') if s.strip()],
    }

USER_AGENTS = [UAOpera, UAFirefox, UAChrome, UAIE8]


def decode_qs(qs, charset='utf-8'):
    '''
    Переводим строку запроса в переменные post/get запроса.
        *qs - строка запроса
        **out_enc - кодировка возвращаемой строки
    '''
    import cgi
    return (u'post=(\n%s\n)' % 
        ',\n'.join(["    (%s, u'%s')" % ("'%s'" % k.strip(),
            v[0].strip().decode(charset))
        for k, v in cgi.parse_qs(qs, keep_blank_values=True).items()]
        )).encode('utf-8')



if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
'''
Base: http://www.jejik.com/articles/2007/02/a_simple_unix_linux_daemon_in_python/
'''
import os
import sys
import time
import atexit
from signal import SIGTERM

import file


class Daemon(object):
    """
    A generic daemon class.
    
    Usage: subclass the Daemon class and override the run() method
    """
    def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = pidfile
    
    def daemonize(self):
        """
        do the UNIX double-fork magic, see Stevens' "Advanced 
        Programming in the UNIX Environment" for details (ISBN 0201563177)
        http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
        """
        try: 
            pid = os.fork() 
            if pid > 0:
                # exit first parent
                sys.exit(0) 
        except OSError, e: 
            sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)
    
        # decouple from parent environment
        #~ os.chdir('/') 
        os.setsid() 
        os.umask(0) 
    
        # do second fork
        try: 
            pid = os.fork() 
            if pid > 0:
                # exit from second parent
                sys.exit(0) 
        except OSError, e: 
            sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1) 

        lock_file = file.lock(self.pidfile)
        if not lock_file:
            sys.stderr.write('Can\'t lock file: %s\n' % self.pidfile)
            sys.exit(1)
        atexit.register(self.delpid)
        lock_file.truncate()
        lock_file.write('%i' % os.getpid())
        lock_file.flush()
        
        # redirect standard file descriptors
        sys.stdout.flush()
        sys.stderr.flush()
        si = open(self.stdin, 'r')
        so = open(self.stdout, 'a+')
        se = open(self.stderr, 'a+', 0)
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())
    
    def delpid(self):
        os.unlink(self.pidfile)

    def start(self):
        """
        Start the daemon
        """
        lock_file = file.lock(self.pidfile)
        if not lock_file:
            sys.stderr.write('Daemon already running\n')
            sys.exit(1)
        # Start the daemon
        lock_file.close()
        self.daemonize()
        self.run()

    def stop(self):
        """
        Stop the daemon
        """
        if not os.path.isfile(self.pidfile):
            sys.stderr.write('Daemon not running\n')
            return
            
        lock_file = file.lock(self.pidfile)
        if lock_file:
            lock_file.close()
            sys.stderr.write('Daemon not running\n')
            return

        try:
            pid = int(open(self.pidfile).read())
        except ValueError:
            sys.stderr.write('Can\'t stop daemon, bad value in file: %s\n' % 
                self.pidfile)
            return

        # Try killing the daemon process	
        try:
            while 1:
                os.kill(pid, SIGTERM)
                time.sleep(0.1)
        except OSError, err:
            err = str(err)
            if err.find("No such process") > 0:
                if os.path.exists(self.pidfile):
                    os.unlink(self.pidfile)
            else:
                print str(err)
                sys.exit(1)

    def restart(self):
        """
        Restart the daemon
        """
        self.stop()
        self.start()

    def run(self):
        """
        You should override this method when you subclass Daemon. It will be called after the process has been
        daemonized by start() or restart().
        """



# -*- coding: utf-8 -*-
import sys


SUBMODULE_OBJECTS = {
    # module : [objects],
    'pp': ['pp'],
    'url': ['url'],
    'daemon': ['Daemon'],
    'debug': ['Timer', 'progress', 'Progress', 'RePrint'],
    'decorators': ['cached_function', 'cached_property', 'cached_classmethod',
        'safely', 'replay', 'single_access'],
    'file': ['file'],
    'gthread': ['gthread'],
    'kv': ['KVBase', 'UKVBase', 'KV', 'KVObject'],
    'memory': ['MemoryTracker'],
    'send': ['send'],
    'structures': ['BigStringList'],
    'thread': ['thread'],
}

ALLOWED = {}
for mod, obj_list in SUBMODULE_OBJECTS.iteritems():
    for obj in obj_list:
        ALLOWED[obj] = mod


class OM(object):
    '''Auto-import'''
    def __getattr__(self, name):
        try:
            mod_name = '%s.%s' % (__name__, ALLOWED[name])
            __import__(mod_name)
            mod = sys.modules[mod_name]
            ret = getattr(mod, name)
        except (KeyError, ImportError, AttributeError):
            raise AttributeError('om has no attribute %s' % name)
        setattr(self, name, ret)
        return ret


om = OM()
    
    


# -*- coding: utf-8 -*-
import array


def main():
    import random
    
    a = BigStringList()
    a.append('Утол')
    a.append('Мар')
    print len(a)
    print a[1]
    print random.choice(a)


class BigStringList(object):
    '''
    Экономичный к памяти список строк
    Использование:
        a = BigStringList()
        a.append('Утол')
        a.append('Мар')
        print len(a)
        print a[1]
        print random.choice(a)
    '''
    def __init__(self):
        self.data = array.array('c')
        self.index = array.array('L')
    
    def append(self, s):
        self.index.append(len(self.data))
        self.data.fromstring(s)
        
    def __getitem__(self, id):
        start = self.index[id]
        try:
            end = self.index[id + 1]
        except IndexError:
            end = len(self.data)
        return self.data[start:end].tostring()
        
    def __len__(self):
        return len(self.index)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
''' Файлы '''

import os
import sys
import random
import shutil
import tempfile


file = sys.modules[__name__]  # for auto-import

def main():
    import om.t
    import time
    
    #~ d = JsonDict('var/tst.json')
    #~ return
    
    #~ CNT = 1024 * 300
    #~ fn = 'var/tst/num_%i.txt' % CNT
    #~ fn = '/media/squash/num_%i.txt' % CNT
    #~ s = 'a' * 1024 + '\n'
    
    #~ cnt = om.t.Cnt(CNT)
    #~ f = open(fn, 'w')
    #~ for i in xrange(CNT):
        #~ f.write(s)
    #~ f.close()
    #~ f = NumIndex(fn)
    #~ f.index()
    
    #~ time.sleep(3)
    #~ f = NumIndex(fn)
    #~ tm = om.t.T()
    #~ for i in xrange(1000):
        #~ f.get_rnd()
        #~ f.fast_rnd_list(100)
    #~ print '%sМ %s' % (CNT / 1024, tm)


    fns = list(dir_fns('.', rec=-1, filter=[
        'isfile',
        #~ 'isdir',
        ]))
    print len(fns)
    print '\n'.join('%5s %s' % (os.path.isfile(fn), fn) for fn in fns)




def lock(filename):
    '''
    Пытается открыть файл в режиме 'r+' и залочить.
    Возвращает файл, если получилось
    '''
    import fcntl
    if not hasattr(lock, '_files'):
        lock._files = {}
    try:
        if not os.path.isfile(filename):
            open(filename, 'w').close()
        file = open(filename, 'r+')
        fcntl.flock(file.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError, ex:
        if ex[0] != 11:
            raise
        return None
    lock._files[filename] = file
    return file

    
def dir_fns(dir, rec=0, filter=None):
    '''
    Все файлы папки
        *dir - папка
        **rec - уровень рекурсивного обхода вложенных папок, -1 = бесконечно
        **filter - что возвращать: ['isfile', 'isdir', 'islink', 'ismount']
    '''
    filter = filter or ['isfile']
    dirs = []
    for fn in os.listdir(dir):
        full_fn = os.path.join(dir, fn)
        if os.path.isdir(full_fn):
            dirs.append(full_fn)
        for el in filter:
            if getattr(os.path, el)(full_fn):
                yield full_fn
                break
    if rec:
        for dir in dirs:
            for fn in dir_fns(dir, rec-1, filter):
                yield fn


class NumIndex(object):
    ''' 
        Огромный спиок строк хранимый в текстовом файле.
        Поддерживает доступ к строке по номеру.
    '''
    
    def __init__(self, fn):
        self.fn = fn
        self.f = open(fn)
        self.ind_len = len(str(os.stat(self.fn)[6]))  # длина индекса
        self.fn_ind = os.path.abspath(fn) + '.om.num.index'
        try:
            self.f_ind = open(self.fn_ind)
            self.count = os.path.getsize(self.fn_ind) / self.ind_len - 1
        except:
            self.count = 0
    
    def index(self):
        '''
        Индексируем файл
        '''
        import tempfile
        
        f_ind = open(tempfile.mkstemp(dir=os.path.dirname(os.path.abspath(self.fn_ind)))[1], 'w')
        self.f.seek(0)

        while True:
            f_ind.write(('%%0%dd' % self.ind_len) % self.f.tell())
            s = self.f.readline()
            if not s:
                break
        f_ind.close()

        os.rename(f_ind.name, self.fn_ind)
        
        self.f_ind = open(self.fn_ind)
        self.count = os.path.getsize(self.fn_ind) / self.ind_len
        
    def get(self, n, cnt=1):
        '''
        Получаем строки начиная с заданой. Нумерация с нуля.
        '''
        if n >= self.count:
            return []

        self.f_ind.seek(n * self.ind_len)
        self.f.seek(int(self.f_ind.read(self.ind_len)))
        
        ret = []
        for i in xrange(cnt):
            s = self.f.readline()
            if not s:
                break
            ret.append(s.rstrip())

        if len(ret) != cnt:
            self.f.seek(0)
            for i in xrange(cnt - len(ret)):
                s = self.f.readline()
                if not s:
                    break
                ret.append(s.rstrip())
        return ret

    def get_rnd(self, cnt=1):
        '''
        Получаем случайные строки
        '''
        n = random.randint(0, self.count - 1)
        if cnt == 1:
            return self.get(n, cnt)[0]
        return self.get(n, cnt)[0]
        
    def fast_rnd_list(self, cnt, seeks=1):
        '''
        Быстрый список случайных строк
            seeks = количество случайных смещений
        '''
        block = cnt / seeks
        res = []
        for i in xrange(seeks):
            res.extend(self.get_rnd(block * 2))
        random.shuffle(res)
        return res[:cnt]


class SList(list):
    ''' 
        Список строк хранимый в текстовом файле.
        При сохранении элеметнов они приводятся к строке.
        Пустые строки при сохранении и загрузке игнорируются.
        Пробельные символы в начале и конце строки удаляются.
    '''
    
    def __init__(self, fn, *args, **kwargs):
        super(SList, self).__init__(*args, **kwargs)
        self.fn = fn
        try:
            self.load()
        except IOError:
            pass
            
    def unique(self):
        '''
        Уникализируем список
        '''
        seen = {} 
        new = [] 
        for item in self: 
            if item in seen: continue 
            seen[item] = 1 
            new.append(item) 
        self.clear().extend(new)
        return self

    def clear(self):
        '''
        Очищаем список
        '''
        del self[:]
        return self

    def load(self):
        '''
        Загружаем список из файла
        '''
        self.clear()
        f = open(self.fn)
        for s in f:
            s = s.strip()
            if not s:
                continue
            self.append(s)
        f.close()
        return self
        
    def save(self):
        '''
        Cохраняем список в файл построчно
        '''
        data = []
        for s in self:
            s = str(s).strip()
            if not s:
                continue
            data.append(s)
        safely_write(self.fn, '\n'.join(data))
        return self

    def append(self, val):
        super(SList, self).append(val)
        return self

    def extend(self, vals):
        super(SList, self).extend(vals)
        return self

    def remove(self, val):
        super(SList, self).remove(val)
        return self

    def reverse(self):
        super(SList, self).reverse()
        return self
        
    def decode(self, enc):
        for i in xrange(len(self)):
            self[i] = self[i].decode(enc)
        return self

    def encode(self, enc):
        for i in xrange(len(self)):
            self[i] = self[i].encode(enc)
        return self


class JsonDict(dict):
    ''' 
    Словарь хранимый в файле в виде json-дампа
        - строки в файле сохраняются в указанной кодировке
        - строковые ключи декодируются
        - при загрузке все строки декодируются
    '''
    def __init__(self, fn, enc='utf-8', *args, **kwargs):
        '''
            *fn - имя файла
            **enc - кодировка файла
        '''
        global json
        import simplejson as json
        
        super(JsonDict, self).__init__(*args, **kwargs)
        self.enc = enc
        self.fn = fn
        try:
            self.load()
        except IOError:
            pass
            
    def _u(self, s):
        if isinstance(s, str):
            s = s.decode(self.enc)
        return s
            
    def __getitem__(self, name):
        return super(JsonDict, self).__getitem__(self._u(name))

    def __setitem__(self, name, val):
        super(JsonDict, self).__setitem__(self._u(name), val)

    def __delitem__(self, name):
        super(JsonDict, self).__delitem__(self._u(name))

    def load(self):
        '''
        Загружаем список из файла
        '''
        self.clear()
        self.update(json.loads(open(self.fn).read().decode(self.enc)))
        return self

    def save(self):
        '''
        Cохраняем список в файл построчно
        '''
        safely_write(self.fn, str(self))
        return self
        
    def __str__(self):
        return json.dumps(dict(self), indent='\t', sort_keys=True,
            ensure_ascii=False).encode(self.enc)


def safely_write(fn, data, make_dirs=True):
    '''
    Безопасная запись в файл (с помощью ренейма)
        *fn - имя файла
        *data - данные для сохранения
    '''
    fd, tmp_fn = tempfile.mkstemp(dir='/tmp')
    tmp = os.fdopen(fd, 'w')
    tmp.write(data)
    tmp.close()
    try:
        shutil.move(tmp_fn, fn)
    except IOError, ex:
        if ex.errno == 2 and make_dirs:
            # создаём папки
            os.makedirs(os.path.dirname(fn))
            try:
                shutil.move(tmp_fn, fn)
            except IOError:
                os.unlink(tmp_fn)
                raise
        else:
            os.unlink(tmp_fn)
            raise

if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Испльзование:

from gevent import monkey; monkey.patch_all()
import om.gthread
import om.url

def print_head(host):
    print 'Starting %s' % host
    r = om.url.query(host)
    print '<-- %s: %s bytes: %r' % (host, len(r.body), r.body[:50])

urls = (h for h in ['www.google.com', 'www.yandex.ru', 'www.python.org', 'dev.pooqle.com'])
om.gthread.run(print_head, urls, 2)

'''
import gevent
from gevent.queue import JoinableQueue


gthread = sys.modules[__name__]  # for auto-import


def main():
    from gevent import monkey; monkey.patch_all()
    import om.url
    
    def print_head(host):
        print 'Starting %s' % host
        r = om.url.query(host)
        print '<-- %s: %s bytes: %r' % (host, len(r.body), r.body[:50])

    urls = ['www.google.com', 'www.yandex.ru', 'www.python.org', 'dev.pooqle.com']
    run(print_head, urls, 2)
    

def run(func, tasks, thread_limit):
    tasks = iter(tasks)
    queue = JoinableQueue()
    for i in xrange(thread_limit):
         gevent.spawn(worker, func, queue, tasks)
    queue.put(tasks.next())
    queue.join()  # block until all tasks are done


def worker(func, queue, tasks):
    while True:
        try:
            queue.put(tasks.next())
        except StopIteration:
            pass
        task = queue.get()
        try:
            func(task)
        finally:
            queue.task_done()


if __name__ == '__main__':
    main()
